/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "tstore.h"

int main(int argc, char ** argv)
{
  uint32_t store_size;
  int store_fd,hash_fd;
  uint8_t buf[512];
  char *namebuf;
  int r;
  uint32_t i;
  int maxused=0;

  if ((argc < 2)||(argc > 3)) {
    fprintf(stderr,"Uage: %s <store> [safetymargin]\n",argv[0]);
    return 1;
  }

  store_fd=open(argv[1],O_RDONLY);
  if (store_fd==-1) {
    perror(argv[1]);
    return 1;
  }

  store_size=test_store_size(store_fd,0);
  store_error(store_size,argv[1]);

  buckets=(store_size+42)/36;
  if (argc==3) {
    uint32_t m=atoi(argv[2]);
    buckets+=m;
  }
  init_hash();

  if (asprintf(&namebuf,"%s.idx",argv[1])<4) {
    perror("Out of memory");
    return 1;
  }

  hash_fd=open(namebuf,O_CREAT|O_RDWR|O_TRUNC,0666);
  if (hash_fd==-1) {
    perror(namebuf);
    return 1;
  }
  ftruncate(hash_fd,buckets*512);

  for (i=1;(r=read(store_fd,buf,512))==512;++i) {
    uint64_t this_hash=hash_sector(buf);
    uint64_t bucket_number=this_hash>>16;
    uint16_t small_hash=this_hash;
    int used;

    if (pread(hash_fd,buf,512,bucket_number*512)!=512) {
      perror(namebuf);
      return 1;
    }

    used=buf[0];
    if (used>maxused) maxused=used;
    if (used==73) {
      fprintf(stderr,"Hash overflow\n");
      return 1;
    }

    PUT_BNUM(buf+(1+7*used),i);
    PUT_HASH(buf+(6+7*used),small_hash);
    buf[0]=used+1;

    if (pwrite(hash_fd,buf,512,bucket_number*512)!=512) {
      perror(namebuf);
      return 1;
    }
  }

  switch(r) {
  default:
    fprintf(stderr,"%s: Incomplete read\n",argv[1]);
    return 1;
  case -1:
    perror(argv[1]);
    return 1;
  case 0:
    break;
  }
  if (i != store_size) {
    fprintf(stderr,"%s: Has %s as we read it\n",
	    argv[1],(i>store_size)?"grown":"shrunk");
    return 1;
  }

  hash_advice(maxused);
  return 0;
}
