/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _TSTORE_H
#define _TSTORE_H

#define _GNU_SOURCE
#define _LARGEFILE_SOURCE  
#define _LARGEFILE64_SOURCE  
#define _FILE_OFFSET_BITS 64

#define _XOPEN_SOURCE 500

#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

uint64_t nextprime(uint64_t p);
uint64_t prevprime(uint64_t p);

extern uint64_t buckets;
void init_hash(void);
uint64_t hash_sector(uint8_t *data);
void hash_advice(int maxused);

uint64_t size_in_blocks(int fd);
uint64_t test_store_size(int fd, int p);
void store_error(uint64_t size, char *name);
ssize_t myread(int fd, void *buf, size_t count);
int filelevels(uint64_t size);

uint64_t get_nbyteint(uint8_t *p, int s);
void put_nbyteint(uint8_t *p, int s, uint64_t n);

#define PUT_BNUM(_p,_v) put_nbyteint(_p,5,_v)
#define GET_BNUM(_p)    get_nbyteint(_p,5)
#define PUT_SIZE(_p,_v) put_nbyteint(_p,7,_v)
#define GET_SIZE(_p)    get_nbyteint(_p,7)
#define PUT_HASH(_p,_v) put_nbyteint(_p,2,_v)
#define GET_HASH(_p)    get_nbyteint(_p,2)

#endif
